/*
 * JBoss, Home of Professional Open Source.
 * Copyright 2011, Red Hat Middleware LLC, and individual contributors
 * as indicated by the @author tags. See the copyright.txt file in the
 * distribution for a full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jboss.as.test.integration.jca.beanvalidation.ra;

/**
 * Connection implementation
 * 
 * @author <a href="mailto:vrastsel@redhat.com">Vladimir Rastseluev</a> 
 */
public class ValidConnectionImpl implements ValidConnection {
    /** ManagedConnection */
    private ValidManagedConnection mc;

    /** ManagedConnectionFactory */
    private ValidManagedConnectionFactory mcf;

    /**
     * Default constructor
     * 
     * @param mc  ManagedConnection
     * @param mcf  ManagedConnectionFactory
     */
    public ValidConnectionImpl(ValidManagedConnection mc, ValidManagedConnectionFactory mcf) {
        this.mc = mc;
        this.mcf = mcf;
    }

    /**
     * Call getResourceAdapterProperty
     * 
     * @return String
     */
    public int getResourceAdapterProperty() {
        return ((ValidResourceAdapter) mcf.getResourceAdapter()).getRaProperty();
    }

    /**
     * Call getManagedConnectionFactoryProperty
     * 
     * @return String
     */
    public String getManagedConnectionFactoryProperty() {
        return mcf.getCfProperty();
    }

    /**
     * Close
     */
    public void close() {
        mc.closeHandle(this);
    }
}
