/*
 * JBoss, Home of Professional Open Source.
 * Copyright 2011, Red Hat, Inc., and individual contributors
 * as indicated by the @author tags. See the copyright.txt file in the
 * distribution for a full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */

package org.jboss.as.test.integration.jpa.hibernate.entity;

import java.util.Set;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToMany;
import javax.persistence.ManyToOne;

/**
 * Flight entity class
 *
 * @author Zbyněk Roubalík
 */
@Entity
public class Flight {

    private Long id;
    private String name;
    private Company company;
    private Set<Customer> customers;

    public Flight(){
    }

    @Id
    @Column(name="flight_id")
    @GeneratedValue(strategy=GenerationType.IDENTITY)
    public Long getId()
    {
       return id;
    }

    public void setId(Long long1)
    {
       id = long1;
    }

    @Column(updatable = false, name = "flight_name", nullable = false, length = 50)
    public String getName()
    {
       return name;
    }

    public void setName(String string)
    {
       name = string;
    }


    @ManyToOne(cascade = {CascadeType.ALL})
    @JoinColumn(name = "comp_id")
    public Company getCompany()
    {
       return company;
    }

    public void setCompany(Company company)
    {
       this.company = company;
    }

    @ManyToMany(cascade = {CascadeType.PERSIST, CascadeType.MERGE}, fetch = FetchType.EAGER)
    public Set<Customer> getCustomers()
    {
       return customers;
    }

    public void setCustomers(Set<Customer> customers)
    {
       this.customers = customers;
    }

}
